#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2023/09/11 10:19:27
@Author  :   ChenHao
@Contact :   jerrychen1990@gmail.com
'''


import logging
import os
from typing import Any

import numpy as np
import zhipuai
from cachetools import LRUCache, cached
from snippets import retry
from agit import AGIT_ENV


from agit.utils import getlog, gen_req_id

logger = getlog(AGIT_ENV, __file__)


def check_api_key(api_key):
    if api_key is None:
        api_key = os.environ.get("ZHIPUAI_API_KEY", None)
    if not api_key:
        raise ValueError("未设置api_key且未设置ZHIPUAI_API_KEY环境变量")
    zhipuai.api_key = api_key


def resp_generator(events, max_len=None, err_resp=None):
    token_left = max_len if max_len else None
    for event in events:
        if event.event == "add":
            if token_left:
                if len(event.data) > token_left:
                    data = event.data[:token_left]
                else:
                    data = event.data
                token_left -= len(data)
                yield data
                if token_left == 0:
                    break
            else:
                yield event.data
        elif event.event == "finish":
            pass
        else:
            logger.error(
                f"zhipu api resp failed with event:{event.event}, data:{event.data}")
            yield err_resp if err_resp else event.data


# sdk请求模型
def call_llm_api(prompt: str, model: str, history=[],
                 api_key=None, stream=True, max_len=None, max_history_len=None,
                 verbose=logging.INFO, max_single_history_len=None,
                 do_search=True, search_query=None,
                 **kwargs) -> Any:
    check_api_key(api_key)

    logger.setLevel(verbose)

    if max_history_len:
        logger.debug(
            f"history length:{len(history)}, cut to {max_history_len}")
        history = history[-max_history_len:]
    if max_single_history_len:
        logger.debug(
            f"cut each history's length to {max_single_history_len}")
        history = [dict(role=e['role'], content=e['content']
                        [:max_single_history_len]) for e in history]
    zhipu_prompt = history + [dict(role="user", content=prompt)]
    total_words = sum([len(e['content']) for e in zhipu_prompt])
    logger.debug(f"zhipu prompt:")
    detail_msgs = []

    for idx, item in enumerate(zhipu_prompt):
        detail_msgs.append(f"[{idx+1}].{item['role']}:{item['content']}")
    logger.debug("\n"+"\n".join(detail_msgs))

    if "ref" not in kwargs:
        ref = dict(enable=do_search, query=search_query if search_query else prompt)
        kwargs.update(ref=ref)
    if "request_id" not in kwargs:
        request_id = gen_req_id(prompt=prompt, model=model)
        kwargs.update(request_id=request_id)
    logger.debug(
        f"{model=}, {stream=}, {kwargs=}, history_len={len(history)}, words_num={total_words}")
    response = zhipuai.model_api.sse_invoke(
        model=model,
        prompt=zhipu_prompt,
        **kwargs
    )
    generator = resp_generator(response.events(), max_len=max_len)
    if stream:
        return generator
    else:
        resp = "".join(list(generator)).strip()
        if max_len:
            resp = resp[:max_len]
        return resp


def call_character_api(prompt: str, user_name, user_info, bot_name, bot_info,
                       history=[], model="characterglm", api_key=None, stream=True, max_len=None, **kwargs):
    check_api_key(api_key)
    zhipu_prompt = history + [dict(role="user", content=prompt)]
    total_words = sum([len(e['content']) for e in zhipu_prompt])
    logger.debug(f"zhipu prompt:")
    detail_msgs = []

    for idx, item in enumerate(zhipu_prompt):
        detail_msgs.append(f"[{idx+1}].{item['role']}:{item['content']}")
    logger.debug("\n"+"\n".join(detail_msgs))
    logger.debug(
        f"{model=},{kwargs=}, history_len={len(history)}, words_num={total_words}")
    meta = {
        "user_info": user_info,
        "bot_info": bot_info,
        "bot_name": bot_name,
        "user_name": user_name
    }
    response = zhipuai.model_api.sse_invoke(
        model=model,
        meta=meta,
        prompt=zhipu_prompt,
        incremental=True,
        ** kwargs
    )
    generator = resp_generator(response.events(), max_len=None)
    if stream:
        return generator
    else:
        resp = "".join(list(generator)).strip()
        if max_len:
            resp = resp[:max_len]
        return resp


@cached(LRUCache(maxsize=1000))
def call_embedding_api(text: str, api_key=None, norm=None, retry_num=2, wait_time=1):
    check_api_key(api_key)

    def attempt():
        resp = zhipuai.model_api.invoke(
            model="text_embedding",
            prompt=text
        )
        if resp["code"] != 200:
            raise Exception(resp["msg"])
        embedding = resp["data"]["embedding"]
        if norm is not None:
            embedding = embedding / np.linalg.norm(embedding, norm)
        return embedding

    if retry_num:
        attempt = retry(retry_num=retry_num, wait_time=wait_time)(attempt)
    return attempt()


if __name__ == "__main__":
    text = "你好"
    resp = "".join(call_llm_api(model="chatglm_turbo", prompt=text))
    print(resp)
    emb = call_embedding_api(text)
    print(len(emb))
    print(emb[:4])



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2023/10/16 10:55:23
@Author  :   ChenHao
@Contact :   jerrychen1990@gmail.com
'''


from chunk import Chunk
import os
import hashlib
import shutil
import time

from typing import List, Optional
from agit import AGIT_ENV, AGIT_KB_DIR

from agit.langchain import (SmartDocumentLoader, SmartTextSplitter,
                            ZhipuAIEmbeddings)
from agit.utils import ConfigMixin
from langchain.docstore.document import Document
from langchain.vectorstores import FAISS
from snippets import log_cost_time, log_function_info, getlog, create_dir_path
from pydantic import Field


logger = getlog(AGIT_ENV, __file__)


class Chunk(Document):
    score: Optional[float] = Field(default=None, description="检索时的相似度")

    @property
    def id(self):
        content = f"{self.page_content}&&{self.metadata['source']}"
        result = hashlib.md5(content.encode())
        return result.hexdigest()

    def __hash__(self) -> int:
        return hash(self.id)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, Chunk) and __value.id == self.id

    def to_str(self, max_content_len=100, max_source_len=50):
        file_name = os.path.basename(self.metadata['source'])

        rs = f"{self.page_content[:max_content_len]} - [{file_name[:max_source_len]}]"
        if self.score is not None:
            rs += f" [{self.score:2.3f}]"
        return rs


class KnowledgeBase(ConfigMixin):
    def __init__(self, name, splitter_config=dict(), index_config=dict()) -> None:

        self.embeddings = ZhipuAIEmbeddings(norm=2)
        self.text_splitter = SmartTextSplitter(**splitter_config)
        self.kb_path = os.path.join(AGIT_KB_DIR, name)
        self.doc_dir = os.path.join(self.kb_path, "documents")
        self.index_dir = os.path.join(self.kb_path, "index")
        self.chunks = set()
        self.index_config = index_config

        if os.path.exists(self.index_dir):
            logger.info(f"loading kb fromt {self.index_dir}")
            self.index = FAISS.load_local(folder_path=self.index_dir,
                                          embeddings=self.embeddings,  **self.index_config)

            # load from index
            self.chunks = set([Chunk(**e.dict())
                               for e in self.index.docstore._dict.values()])

        else:
            logger.info(f"{self.index_dir} not exists, will not load kb")
            self.index = None

    @log_cost_time(name="rebuild")
    def rebuild(self):
        logger.info(f"rebuild index for {self.kb_path}")
        self.chunks = set()
        self.index = None
        for file in os.listdir(self.doc_dir):
            self.update_document(doc_path=os.path.join(self.doc_dir, file))
        self.store()

    def load_and_cut(self, doc_path, max_page=None) -> List[Chunk]:
        loader = SmartDocumentLoader(doc_path, max_page=max_page)
        chunks = loader.load()
        chunks = self.text_splitter.split_documents(chunks)
        chunks = [Chunk(**e.dict()) for e in chunks]
        logger.info(f"split {len(chunks)} chunks from {doc_path}")
        return chunks

    def _deduplicate(self, chunks: List[Chunk]):
        logger.info("deduplicating chunks")
        rs_chunks = []

        for chunk in chunks:
            if chunk in self.chunks:
                continue
            rs_chunks.append(chunk)
        logger.info(f"get {len(rs_chunks)} uniq chunks from {len(chunks)} origin chunks")
        return rs_chunks

    # TODO 优雅的update的实现方式
    def update_document(self, doc_path: str, max_page=None):
        logger.info(f"updating file :{doc_path}")
        chunks = self.load_and_cut(doc_path, max_page=max_page)
        chunks = self._deduplicate(chunks)
        ids = [e.id for e in chunks]
        if not chunks:
            return
        if not self.index:
            logger.debug(f"create index")
            self.index = FAISS.from_documents(chunks, embedding=self.embeddings, ids=ids, **self.index_config)
        else:
            logger.debug(f"add {len(chunks)} chunks to {self.index}")
            self.index.add_documents(
                chunks, embedding=self.embeddings, ids=ids,  **self.index_config)
        self.chunks |= set(chunks)

        logger.info("add documents done")

    # 添加新的文档
    def add_document(self, doc_path: str, max_page=None):
        tgt_path = os.path.join(self.doc_dir, os.path.basename(doc_path))
        logger.info(f"copy document {doc_path} to {tgt_path}")
        shutil.copy(doc_path, tgt_path)
        self.update_document(tgt_path, max_page=max_page)

    def learn(self, knowledge: str):
        doc_path = os.path.join(
            self.doc_dir, "learn-" + time.strftime('%Y-%m-%d', time.localtime())+".txt")

        create_dir_path(doc_path)
        with open(doc_path, "a", encoding="utf-8") as f:
            f.write(knowledge)
            f.write("\n")

        logger.debug(f"learning knowledge in {doc_path}")
        self.update_document(doc_path)

    def store(self):
        assert self.index
        logger.info(f"store to {self.index_dir}")
        self.index.save_local(folder_path=self.index_dir)

    @property
    def chunk_num(self):
        return len(self.chunks)

    @log_cost_time(name="kb_search")
    @log_function_info()
    def search(self, query, top_k=4, threshold=None) -> List[Chunk]:
        if not self.index:
            logger.warning("kb is empty, return []")
            return []

        records = self.index.similarity_search_with_score(
            query, k=top_k, score_threshold=threshold)
        logger.info(f"{len(records)} records found")
        chunks = [(Chunk(**d.dict(), score=s)) for d, s in records]
        return chunks



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2023/10/18 15:53:39
@Author  :   ChenHao
@Contact :   jerrychen1990@gmail.com
'''


from typing import List, Tuple

from cachetools import LRUCache, cached
from tqdm import tqdm
from agit.backend.zhipuai_bk import call_embedding_api
from agit.utils import cal_vec_similarity
from snippets import getlog
from agit import AGIT_ENV

logger = getlog(AGIT_ENV, __file__)


@cached(LRUCache(maxsize=1000))
def get_cand_embeddings(cands: Tuple):
    rs = dict()
    logger.info("calculating cand embeddins")
    for cls, desc in tqdm(cands):
        emb = call_embedding_api(text=desc, norm=2)
        rs[cls] = emb
    return rs


def cls_by_embedding(query: str, cands: dict) -> dict:
    """
    根据query和cands的embedding计算相似度, 从而达到分类的效果
    """
    query_emb = call_embedding_api(text=query, norm=2)
    cand_embs = get_cand_embeddings(tuple(cands.items()))

    resp = []
    for cls, cand_emb in tqdm(cand_embs.items()):
        sim = cal_vec_similarity(query_emb, cand_emb, normalize=False, metric="cosine")
        resp.append((cls, sim))
    resp.sort(key=lambda x: x[1], reverse=True)
    sum_score = sum(e[1] for e in resp)
    resp = [dict(cls=cls, prob=score/sum_score, score=score) for cls, score in resp]
    return resp


if __name__ == "__main__":
    query = "讲一个笑话"
    cands = {
        "chat-funny": "有趣的内容，包括笑话、相声、段子",
        "chat-other": "普通的闲聊对话",
        "chat-idom": "成语接龙",
    }
    resp = cls_by_embedding(query, cands)
    logger.info(resp)
    resp = cls_by_embedding(query, cands)
    logger.info(resp)

    query = "讲一个笑话"
    cands = {
        "chat-funny": "讲一个笑话",
        "chat-other": "普通的闲聊对话",
        "chat-idom": "成语接龙",
    }
    resp = cls_by_embedding(query, cands)
    logger.info(resp)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2023/11/01 15:27:35
@Author  :   ChenHao
@Contact :   jerrychen1990@gmail.com
'''

from typing import Callable, List
from pydantic import BaseModel
import tushare as ts
from agit import AGIT_ENV
from snippets.logs import getlog

logger = getlog(AGIT_ENV, __file__)


class Parameter(BaseModel):
    name: str
    type: str
    description: str
    required: bool


class Tool(BaseModel):
    name: str
    description: str
    parameters: List[Parameter]
    func: Callable


def track_stock(symbol):
    try:
        df = ts.get_realtime_quotes(symbol)
        return dict(price=df["price"][0])
    except Exception as e:
        logger.error(e)
        return {}


TRACK_STOCK = Tool(name="track_stock", description="追踪指定股票的实时价格", func=track_stock,
                   parameters=[Parameter(name="symbol", type="str", description="股票代码", required=True)])

ALL_TOOLS = [TRACK_STOCK]

_TOOL_DICT = {t.name: t for t in ALL_TOOLS}


def get_tool(tool_name: str) -> Tool:
    if tool_name not in _TOOL_DICT:
        raise Exception(f"tool {tool_name} not found")
    return _TOOL_DICT[tool_name]


if __name__ == "__main__":
    print(track_stock("000001"))



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2023/09/11 10:55:05
@Author  :   ChenHao
@Contact :   jerrychen1990@gmail.com
'''
import os
from typing import List

import openai
import logging
from agit import AGIT_ENV
from agit.utils import getlog
from openai import OpenAI


logger = getlog(AGIT_ENV, __file__)


def get_client(api_key, base_url):
    if api_key is None:
        api_key = os.environ.get("OPENAI_API_KEY", None)
    if not api_key:
        raise ValueError("api_key is required")
    return OpenAI(api_key=api_key, base_url=base_url)


def get_gen(chunks):
    for chunk in chunks:
        # logger.debug(chunk)
        content = chunk.choices[0].delta.content
        yield content or ""


def call_llm_api(prompt, model="gpt-3.5-turbo-1106",  history=[],
                 base_url=None,
                 system: str = None, tools: List = None, role="user",
                 stream=True, api_key=None, api_base=None,
                 verbose=logging.INFO, **kwargs):

    def _build_messages(prompt, history, system, tools):
        messages = history + [dict(role=role, content=prompt)]
        if system:
            messages = [dict(role="system", content=system, tools=tools)] + messages
        return messages

    messages = _build_messages(prompt, history, system, tools)
    logger.setLevel(verbose)

    client = get_client(api_key=api_key, base_url=base_url)

    detail_msgs = []
    for idx, item in enumerate(messages):
        msg = f"[{idx+1}].{item['role']}:{item['content']}"
        if "tools" in item:
            msg += (f"\n[tools]:{tools}")

        detail_msgs.append(msg)
    logger.debug("\n"+"\n".join(detail_msgs))

    logger.info(f"request to openai with {model=}, {kwargs=}")

    chunks = client.chat.completions.create(
        model=model,
        messages=messages,
        stream=True,
    )
    gen = get_gen(chunks)
    if stream:
        return gen
    return "".join(gen)


if __name__ == "__main__":
    text = "你好"
    resp = call_llm_api(text, stream=False, verbose=logging.DEBUG)
    print(resp)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2023/10/16 10:56:19
@Author  :   ChenHao
@Contact :   jerrychen1990@gmail.com
'''


from lib2to3 import pytree
import os

from typing import List, Union, Generator
from agit import AGIT_ENV, assistant_default_prompt_template, AGIT_TEMP_DIR

from agit.backend import call_llm_api
from agit.tool import Tool, get_tool
from agit.utils import ConfigMixin
from snippets import getlog, jdumps
from pydantic import BaseModel
from agit.kb import Chunk, KnowledgeBase


logger = getlog(AGIT_ENV, __file__)


class AssistantResp(BaseModel):
    content: Union[str, Generator]
    references: List = []


class Assistant(ConfigMixin):
    def __init__(self, name: str,
                 llm_model="chatglm_lite",
                 prompt_template=assistant_default_prompt_template,
                 tmp_dir=None,
                 system=None,
                 tools: List[Union[str, Tool]] = [],
                 kb_config=dict()) -> None:
        self.name = name
        self.llm_model = llm_model
        self.history = []
        self.prompt_template = prompt_template
        self.system = system
        self._init_tools(tools)

        if "name" not in kb_config:
            kb_config["name"] = f"{self.name}_kb"
        if not tmp_dir:
            tmp_dir = os.path.join(AGIT_TEMP_DIR, "temp_knowledge")
        self.tmp_dir = tmp_dir
        self.kb = KnowledgeBase.from_config(kb_config)

    def _init_tools(self, tools: List[Union[str, Tool]]):
        self.tools: List[Tool] = []
        for tool in tools:
            if isinstance(tool, str):
                try:
                    tool = get_tool(tool)
                    self.tools.append(tool)
                except Exception as e:
                    print(e)
                    pass
            else:
                self.tools.append(tool)

    def __call__(self, *args, **kwargs):
        return self.chat(*args, **kwargs)

    def _build_prompt(self, message: str, chunks: List[Chunk]) -> str:
        prompt = self.prompt_template.replace("{{message}}", message)
        reference = ""
        for idx, chunk in enumerate(chunks):
            reference += f"{idx+1}.{chunk.page_content}"
            reference += "\n\n"

        prompt = prompt.replace("{{reference}}", reference)
        return prompt

    def _build_tool_info(self):
        return [t.model_dump(exclude={"func"}) for t in self.tools]

    def chat(self, message: str, stream=False, use_history=True, llm_model=None,
             use_tool=False, use_kb=True,
             temperature=0.7, top_k=4, threshold=None, **kwargs) -> AssistantResp:

        if use_kb:
            chunks = self.kb.search(
                message, top_k=top_k, threshold=threshold)
            logger.info("references details")
            for idx, chunk in enumerate(chunks):
                logger.debug(f"[{idx+1}] - {chunk}")
            prompt = self._build_prompt(message, chunks)

        else:
            chunks = []
            prompt = message

        tools = self._build_tool_info() if use_tool else None

        resp = call_llm_api(prompt=prompt, model=llm_model if llm_model else self.llm_model,
                            history=self.history if use_history else [],
                            system=self.system, tools=tools,
                            verbose=logger.level,  temperature=temperature, stream=stream,
                            do_search=True, search_query=message, **kwargs)

        self.history.append(dict(role="user", content=message))
        if isinstance(resp, str):
            self.history.append(dict(role="assistant", content=resp))

        tool_name, kwargs = self.tool_parse(resp)
        logger.info(f"{tool_name=}, {kwargs=}")

        if tool_name:
            tool_resp = self.tool_call(tool_name, **kwargs)
            logger.info(f"tool_resp: {tool_resp}")
            if tool_resp:
                message = jdumps(tool_resp)
                resp = self.chat(message=message, stream=stream, use_history=use_history,
                                 use_tool=use_tool, use_kb=use_kb, temperature=0.7, top_k=4, threshold=None, **kwargs, role="observation")
                return resp

        # TODO resp是迭代器时的特殊处理

        # logger.info(f"chunks: {chunks}")
        resp = AssistantResp(content=resp, references=chunks)
        return resp

    def tool_call(self, tool_name, *args, **kwargs):
        logger.debug(f"try to call {tool_name} with {kwargs=}")

        tool_map = {t.name: t.func for t in self.tools}
        if tool_name not in tool_map:
            logger.warning(f"tool not found: {tool_name}")
            resp = {}
        else:
            func = tool_map[tool_name]
            resp = func(*args, **kwargs)
        return resp

    def learn(self, knowledge_or_path: str):
        if os.path.exists(knowledge_or_path):
            self.kb.add_document(knowledge_or_path)
        else:
            self.kb.learn(knowledge_or_path)

    def store(self):
        import re
        re.DT
        re.findall
        self.kb.store()

    @property
    def chunk_num(self):
        return self.kb.chunk_num

    def search_db(self, *args, **kwargs):
        return self.kb.search(*args, **kwargs)

    def clear_history(self):
        self.history = []

    def add_document(self, doc_path, **kwargs):
        self.kb.update_document(doc_path, **kwargs)
        self.kb.store()

    # TODO 暴露这个接口不安全
    def add_resp(self, resp: str):
        self.history.append(dict(role="assistant", content=resp))

    def add_tool(self, tool: Tool):
        self.tools.append(tool)

    def tool_parse(self, content: str):
        import re
        pattern = '''(.*?)\s+```python\s+tool_call(.*?)\s+```'''
        for item in re.findall(pattern, content, re.DOTALL):
            func_name, kwargs = item
            try:
                kwargs = "dict"+kwargs
                kwargs = eval(kwargs)
                func_name, kwargs
                return func_name, kwargs
            except Exception as e:
                pass
        return None, None



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2023/09/13 10:40:06
@Author  :   ChenHao
@Contact :   jerrychen1990@gmail.com
'''

import logging

from langchain.docstore.document import Document
from langchain.document_loaders.base import BaseLoader

logger = logging.getLogger(__name__)


class SmartDocumentLoader(BaseLoader):
    def __init__(self, file_path: str, max_page=None):
        self.read_func = {
            "html": self.read_html,
            "htm": self.read_html,
            "txt": self.read_text,
            "pdf": self.read_pdf
        }
        self.file_path = file_path
        self.max_page = max_page

    def read_text(self, path: str) -> str:
        with open(path, "r", encoding="utf-8") as f:
            content = f.read()
        return content

    def read_html(self, path: str) -> str:
        content = self.read_text(path=path)
        import html2text
        h = html2text.HTML2Text()
        h.ignore_links = True
        h.ignore_images = True
        h.mark_code = True
        text = h.handle(content)
        return text
    
    def read_pdf(self, path:str) -> str:
        import PyPDF2
        with open(path, "rb") as f:
            pdf_reader = PyPDF2.PdfReader(f)
            logger.debug(f"got {len(pdf_reader.pages)} pages")
            pages = pdf_reader.pages
            if self.max_page:
                pages = pages[:self.max_page]
            rs = ""    
            for page in pages:
                rs+=page.extract_text()
            
        return rs


        


        


    def load(self):
        surfix = self.file_path.split(".")[-1]
        if surfix not in self.read_func:
            logger.warning(
                f"unsupported file suffix: {surfix}, try to read as txt")
            func = self.read_text
        func = self.read_func[surfix]
        content = func(self.file_path)
        metadata = {"source": self.file_path}
        return [Document(page_content=content, metadata=metadata)]



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2023/09/13 10:36:06
@Author  :   ChenHao
@Contact :   jerrychen1990@gmail.com
'''
import re
from typing import List

from langchain.text_splitter import TextSplitter
from numpy import maximum
from snippets import batchify, flat


def _cut2maxlen(text, max_len):
    if len(text) < max_len:
        return [text]
    return ["".join(e) for e in batchify(text, max_len)]


def merge(pieces: list[str], joiner=",", min_len=10, max_len=100, overlap=0.3):
    pieces = flat([_cut2maxlen(e, max_len)for e in pieces])
    result = []

    idx = 0
    acc = []
    sum_len = 0
    while idx < len(pieces):
        p = pieces[idx]
        if not acc:
            acc.append(p)
            sum_len += len(p)
            idx += 1
        else:
            if sum_len + len(p) > max_len:
                result.append(joiner.join(acc))
                acc.clear()
                sum_len = 0
            else:
                acc.append(p)
                sum_len += len(p)
                idx += 1
    if acc:
        result.append(joiner.join(acc))
    return result


class SmartTextSplitter(TextSplitter):
    def __init__(self, delimer="[,，。?\？！!\n]|\.{3+}", joiner=",", min_len=10, max_len=100, overlap=0.3):
        super().__init__()
        self.delimer = delimer
        self.joiner = joiner
        self.min_len = min_len
        self.max_len = max_len
        self.overlap = overlap

    def split_text(self, text: str) -> List[str]:
        pieces = re.split(self.delimer, text)
        pieces = [e.strip() for e in pieces if e.strip()]
        merged = merge(pieces, joiner=self.joiner, min_len=self.min_len,
                       max_len=self.max_len, overlap=self.overlap)
        return merged


if __name__ == "__main__":
    text = "你好，哈哈哈，这是一段很长很长很长很长很长很长很长很长很长很长很长很长很长很长的 文本呀，啦啦啦"
    text = "wifi密码是智谱123456,办公地址是EFC英国中心T2 32楼"
    splitter = SmartTextSplitter()
    pieces = splitter.split_text(text, min_len=5, max_len=20, overlap=0.3)
    for p in pieces:
        print(p)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2023/08/15 11:11:03
@Author  :   ChenHao
@Contact :   jerrychen1990@gmail.com
'''
import os

__version__ = "0.1.0"
AGIT_ENV = os.environ.get("AGIT_ENV", "local")
AGIT_PROJECT_HOME = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
AGIT_DATA_HOME = os.environ.get("AGIT_DATA_HOME", AGIT_PROJECT_HOME)


AGIT_TEMP_DIR = os.path.join(AGIT_DATA_HOME, "temp")
AGIT_KB_DIR = os.path.join(AGIT_DATA_HOME, "kb")
AGIT_CONFIG_DIR = os.path.join(AGIT_DATA_HOME, "config")


assistant_default_prompt_template = '''你是一个知识渊博、风趣幽默的人工智能小助手
你回答问题的时候可以参考reference标志的信息
reference
```
{{reference}}
```
问题:
{{message}}'''

LLM_MODELS = [
    "chatglm_lite",
    "chatglm_std",
    "chatglm_pro",
    "gpt-3.5-turbo-0301"
]



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2023/09/13 10:25:05
@Author  :   ChenHao
@Contact :   jerrychen1990@gmail.com
'''


from typing import List

from langchain.embeddings.base import Embeddings
import numpy as np
from pydantic import BaseModel
from tqdm import tqdm
from agit import AGIT_ENV

from agit.backend.zhipuai_bk import call_embedding_api
from agit.utils import getlog

logger = getlog(AGIT_ENV, __file__)


class ZhipuAIEmbeddings(BaseModel, Embeddings):
    norm: int = None

    def embed_documents(self, texts: List[str]) -> List[List[float]]:
        rs = []
        for text in tqdm(texts):
            emb = call_embedding_api(text=text, norm=self.norm)
            logger.debug(f"embedding of chunk:{text}, len:{len(emb)}, l2:{np.linalg.norm(emb)}")

            rs.append(emb)
        return rs

    def embed_query(self, text: str) -> List[float]:
        emb = call_embedding_api(text, norm=self.norm)
        logger.debug(f"embedding of query:{text}, len:{len(emb)}, l2:{np.linalg.norm(emb)}")

        return emb



import agit.backend.openai_bk as openai_bk
import agit.backend.zhipuai_bk as zhipuai_bk


def call_llm_api(prompt: str, model: str, **kwargs):
    if "chatglm" in model:
        if model == "chatglm3-6b":
            return openai_bk.call_llm_api(prompt=prompt, model=model, **kwargs)
        return zhipuai_bk.call_llm_api(prompt=prompt, model=model, **kwargs)
    else:
        return openai_bk.call_llm_api(prompt=prompt, model=model, **kwargs)
